    function [xCR, plotresCR, ploterrCR,alpha,beta,gamma] = ...
        cr_poly(A, rhs, x0, n_max, tol, matvec, x_star)
%   function [xCR, plotresCR, ploterrCR] = ...
%       cr_poly(A, rhs, x0, n_max, tol, matvec, x_star)
%   BF 13.07.99
%
%   computes the solution of Ax = rhs using the CR scheme.
% 
%   startvector:                      x0
%   maximal number of iterations:     n_max
%   desired accuracy of the residual: tol
%
%   uses: matvec
%

    %   initialize
    n               =   1;
    N               =   length(x0);
    xCR             =   x0;
    rCR             =   rhs - feval( matvec, A, xCR );
    rCR_ArCR        =   1;
    norm_rCR        =   norm(rCR);
    norm_r0         =   norm_rCR;
    norm_e0         =   norm( x_star - x0 );
    wCR             =   zeros(N, 1);
    AwCR            =   zeros(N, 1);
    plotresCR       =   zeros(n_max, 1);
    plotresCR(n)    =   log10( norm_rCR / norm_r0 );
    ploterrCR       =   zeros(n_max, 1);
    ploterrCR(n)    =   log10( norm( x_star - xCR ) / norm_e0 );

    ArCR=feval(matvec,A,rCR); 
    

    %   iterate
    while (n < n_max + 1) &  ( norm_rCR / norm_r0 > tol ),

        n               =   n + 1;
        rCR_ArCR_old    =   rCR_ArCR;
        ArCR            =   feval( matvec, A, rCR );
        rCR_ArCR        =   rCR' * ArCR;

        nu              =   rCR_ArCR / rCR_ArCR_old;
        wCR             =   nu * wCR - rCR;
        AwCR            =   nu * AwCR - ArCR;
        eta             =   - rCR_ArCR / ( AwCR' * AwCR );

        %   update

        xCR             =   xCR + eta * wCR;
        rCR             =   rCR - eta * AwCR;
        %norm_rCR        =   norm(rCR);
 
	%   three-term recurrence coefficients
	
	alpha(n-1)=(ArCR'*ArCR)/(rCR_ArCR);
	if n==2
           beta(1)=0;
	   gamma(1)=-alpha(1);
	 else
	       beta(n-1)=gamma(n-2)*(rCR_ArCR)/(rCR_ArCR_old);
          gamma(n-1)=-(alpha(n-1)+beta(n-1));
	end
        norm_rCR        =   norm( rhs - feval( matvec, A, xCR ) );
        plotresCR(n)    =   log10( norm_rCR / norm_r0 );
        ploterrCR(n)    =   log10( norm( x_star - xCR ) / norm_e0 );

        if rem(n, 5) == 0,  disp([n, norm_rCR / norm_r0]);  end;

    end;    %   while

    plotresCR       =   plotresCR(1:n);
    ploterrCR       =   ploterrCR(1:n);

return; 





